<?php
/**
 * A general user class 
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class BaseUser extends BaseFrameworkSystem {
	/**
	 * Username of current user
	 */
	private $userName = "";

	/**
	 * Email of current user
	 */
	private $email = "";

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * "Getter" for databse entry
	 *
	 * @return	$entry	An array with database entries
	 * @throws	NullPointerException	If the database result is not found
	 * @throws	InvalidDatabaseResultException	If the database result is invalid
	 */
	private function getDatabaseEntry () {
		// Is there an instance?
		if (is_null($this->getResultInstance())) {
			// Throw new exception
			throw new NullPointerException($this, self::EXCEPTION_IS_NULL_POINTER);
		} // END - if

		// Rewind it
		$this->getResultInstance()->rewind();

		// Do we have an entry?
		if (!$this->getResultInstance()->valid()) {
			throw new InvalidDatabaseResultException(array($this, $this->getResultInstance()), DatabaseResult::EXCEPTION_INVALID_DATABASE_RESULT);
		} // END - if

		// Get next entry
		$this->getResultInstance()->next();

		// Fetch it
		$entry = $this->getResultInstance()->current();

		// And return it
		return $entry;
	}

	/**
	 * Setter for username
	 *
	 * @param	$userName	The username to set
	 * @return	void
	 */
	public final function setUserName ($userName) {
		$this->userName = $userName;
	}

	/**
	 * Setter for email
	 *
	 * @param	$email	The email to set
	 * @return	void
	 */
	protected final function setEmail ($email) {
		$this->email = $email;
	}

	/**
	 * Getter for username
	 *
	 * @return	$userName	The username to get
	 */
	public final function getUsername () {
		return $this->userName;
	}

	/**
	 * Getter for email
	 *
	 * @return	$email	The email to get
	 */
	public final function getEmail () {
		return $this->email;
	}

	/**
	 * Determines wether the username exists or not
	 *
	 * @return	$exists		Wether the username exists
	 */
	public function ifUsernameExists () {
		// By default the username does not exist
		$exists = false;

		// Is a previous result there?
		if (is_null($this->getResultInstance())) {
			// Get a UserDatabaseWrapper instance
			$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper_class');

			// Create a search criteria
			$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

			// Add the username as a criteria and set limit to one entry
			$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_USERNAME, $this->getUsername());
			$criteriaInstance->setLimit(1);

			// Get a search result
			$this->setResultInstance($wrapperInstance->doSelectByCriteria($criteriaInstance));
		} else {
			// Rewind it
			$this->getResultInstance()->rewind();
		}

		// Search for it
		if ($this->getResultInstance()->next()) {
			// Entry found
			$exists = true;
		} // END - if

		// Return the status
		return $exists;
	}

	/**
	 * Determines wether the email exists or not
	 *
	 * @return	$exists		Wether the email exists
	 */
	public function ifEmailAddressExists () {
		// By default the email does not exist
		$exists = false;

		// Is a previous result there?
		if (is_null($this->getResultInstance())) {
			// Get a UserDatabaseWrapper instance
			$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper_class');

			// Create a search criteria
			$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

			// Add the username as a criteria and set limit to one entry
			$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_EMAIL, $this->getEmail());
			$criteriaInstance->setLimit(1);

			// Get a search resultInstance
			$this->setResultInstance($wrapperInstance->doSelectByCriteria($criteriaInstance));
		} else {
			// Rewind it
			$this->getResultInstance()->rewind();
		}

		// Search for it
		if ($this->getResultInstance()->next()) {
			// Entry found
			$exists = true;
		} // END - if

		// Return the status
		return $exists;
	}

	/**
	 * Checks if the supplied password hash in request matches with the stored
	 * in database.
	 *
	 * @param	$requestInstance	A requestable class instance
	 * @return	$matches			Wether the supplied password hash matches
	 */
	public function ifPasswordHashMatches (Requestable $requestInstance) {
		// By default nothing matches... ;)
		$matches = false;

		// Get a UserDatabaseWrapper instance
		$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper_class');

		// Create a search criteria
		$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

		// Add the username as a criteria and set limit to one entry
		$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_USERNAME, $this->getUserName());
		$criteriaInstance->setLimit(1);

		// Get a search resultInstance
		$this->setResultInstance($wrapperInstance->doSelectByCriteria($criteriaInstance));

		// Search for it
		if ($this->getResultInstance()->next()) {
			// Get the current entry (can only be one!)
			$entry = $this->getResultInstance()->current();

			// So does the hashes match?
			//* DEBUG: */ echo $requestInstance->getRequestElement('pass_hash')."/".$entry['pass_hash'];
			$matches = ($requestInstance->getRequestElement('pass_hash') === $entry['pass_hash']);
		} // END - if

		// Return the status
		return $matches;
	}

	/**
	 * "Getter" for user's password hash
	 *
	 * @return	$passHash	User's password hash from database result
	 */
	public final function getPasswordHash () {
		// Default is missing password hash
		$passHash = null;

		// Get a database entry
		$entry = $this->getDatabaseEntry();

		// Is the password hash there?
		if (isset($entry['pass_hash'])) {
			// Get it
			$passHash = $entry['pass_hash'];
		}

		// And return the hash
		return $passHash;
	}

	/**
	 * Getter for field name
	 *
	 * @param	$fieldName		Field name which we shall get
	 * @return	$fieldValue		Field value from the user
	 * @todo	Do we need to secure this here against missing results?
	 */
	public final function getField ($fieldName) {
		// Default field value
		$fieldValue = null;

		// Get current array
		$fieldArray = $this->getResultInstance()->current();

		// Does the field exist?
		if (isset($fieldArray[$fieldName])) {
			// Get it
			$fieldValue = $fieldArray[$fieldName];
		} // END - if

		// Return it
		return $fieldValue;
	}

	/**
	 * Getter for primary key value
	 *
	 * @return	$primaryValue	Value of the primary key based on database type
	 */
	public final function getPrimaryKey () {
		// Get a user database wrapper
		$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper_class');

		// Get the primary key back from the wrapper
		$primaryKey = $wrapperInstance->getPrimaryKeyValue();

		// Get that field
		$primaryValue = $this->getField($primaryKey);

		// Return the value
		return $primaryValue;
	}
}

// [EOF]
?>
